local SW = LibStub("AceAddon-3.0"):NewAddon("StackWatch", "AceEvent-3.0", "AceConsole-3.0")

local SW_RI_STAR = 1;
local SW_RI_CIRCLE = 2;
local SW_RI_DIAMOND = 3;
local SW_RI_TRIANGLE = 4;
local SW_RI_MOON = 5;
local SW_RI_SQUARE = 6;
local SW_RI_CROSS = 7;
local SW_RI_SKULL = 8;

local SW_RT2STRING = {"Star", "Circle", "Diamond", "Triangle", "Moon", "Square", "Cross", "Skull" };

function SW:OnInitialize()
    -- Called when the addon is loaded
	SW._units = {};
	SW._usedIcons = {};
	SW._playerIcons = {};
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("StackWatch", SW_SLASH_COMMANDS );
	
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("StackWatch", "StackWatch" );
	SW_SAVED_VARIABLES = LibStub("AceDB-3.0"):New("StackWatch", SW_SAVED_VARIABLES, "Default");
	
	self:SetEnabledState( SW_SAVED_VARIABLES.profile.enabled );
end

function SW:OnEnable()
    -- Called when the addon is enabled
    if( SW_SAVED_VARIABLES.profile.enabled ) then
		SW._units = {};
		SW._usedIcons = {};
		SW._playerIcons = {};
	    
	    self:RegisterEvent( "COMBAT_LOG_EVENT_UNFILTERED" );
	    self:RegisterEvent( "PLAYER_REGEN_DISABLED" );
	    self:RegisterEvent( "PLAYER_REGEN_ENABLED" );
	    self:SetEnabledState( true );
	end;
end;

function SW:OnDisable()
    -- Called when the addon is disabled
    self:UnregisterAllEvents();
end;



function SW:COMBAT_LOG_EVENT_UNFILTERED( ... )
	local playerName = select( 8, ... );
	
	if( UnitInParty( playerName ) == 1 or UnitInRaid( playerName ) ~= nil ) then
		local eventType = select( 3, ... );
		
	
		local auraType = select( 11, ... ) or "";
		local auraAmmount = select( 12, ... ) or "";
		

		if( eventType == "SPELL_AURA_APPLIED" and auraType == self:GetDebuff() ) then
			--Aura first applied
			SW:SetAuraCount( playerName, 1 );
			local count = self:GetAuraCount( playerName );
			if( count >= self:GetThreshold() and self:GetThreshold() > 0) then
				self:NotifyPlayer( playerName );
			end;
		elseif( eventType == "SPELL_AURA_APPLIED_DOSE" and auraType == self:GetDebuff() ) then
			--Stack applied

			self:AddAuraCount( playerName );
			local count = self:GetAuraCount( playerName );

			if( count >= self:GetThreshold() and self:GetThreshold() > 0) then
				self:NotifyPlayer( playerName );
			end;
		elseif( eventType == "SPELL_AURA_REMOVED" and auraType == self:GetDebuff() ) then
			self:ClearAuraCount( playerName );
			self:ClearIcon( playerName )
		end;
	end;
end;

--enter combat
function SW:PLAYER_REGEN_DISABLED()
	self:Reset();
end;

--leave combat
function SW:PLAYER_REGEN_ENABLED()
	
end;


function SW:AssignIcon( playerName )
--    0 = no icon 
--    1 = Yellow 4-point Star 
--    2 = Orange Circle 
--    3 = Purple Diamond 
--    4 = Green Triangle 
--    5 = White Crescent Moon 
--    6 = Blue Square 
--    7 = Red "X" Cross 
--    8 = White Skull 
	local icons = self:GetEnabledIcons();
	
	
	if( not SW._playerIcons[playerName] ) then
		for i,isIconUsable in ipairs(icons) do
			if( isIconUsable and not SW._usedIcons[i] ) then
				SW._usedIcons[i] = playerName;
				SW._playerIcons[playerName] = i;
				SetRaidTarget( playerName, i );
				break;
			end;
		end;
	end;
end;

function SW:ClearIcon( playerName )
	local iconIndex = SW._playerIcons[playerName]
	
	if( iconIndex ) then
		SW._playerIcons[playerName] = nil;
		SW._usedIcons[iconIndex] = nil;
	end;
	
	SetRaidTarget( playerName, 0 );
end;


function SW:GetEnabledIcons()
	return SW_SAVED_VARIABLES.profile.iconsUsed;
end;


function SW:IsIconEnabled( iconIndex )
	return SW_SAVED_VARIABLES.profile.iconsUsed[iconIndex] or false;
end;

function SW:SetIsIconEnabled( iconIndex, isEnabled, isSlashCommand )
	if( isSlashCommand ) then
		if( isEnabled ) then
			self:Print( string.format( "%s enabled", SW_RT2STRING[iconIndex] ) );
		else
			self:Print( string.format( "%s disabled", SW_RT2STRING[iconIndex]) );
		end;
	end;

	SW_SAVED_VARIABLES.profile.iconsUsed[iconIndex] = isEnabled;
end;

function SW:NotifyPlayer( playerName )
	self:AssignIcon( playerName );
	SendChatMessage( (SW_SAVED_VARIABLES.profile.messagePrefix or " " ) .. string.format(  " %s", (SW_SAVED_VARIABLES.profile.watchedBuff or "-none-") ), "WHISPER", nil, playerName );
end;


function SW:SetAuraCount( playerName, count )
	count = tonumber(count) or 0;
	SW._units[playerName] = count;
end;

function SW:SetMessagePrefix( prefix, isSlashCommand )
    SW_SAVED_VARIABLES.profile.messagePrefix = prefix or "";
end;

function SW:GetMessagePrefix()
    return SW_SAVED_VARIABLES.profile.messagePrefix;
end;

function SW:AddAuraCount( playerName, count )
	count = count or 1;
	if( not SW._units[playerName] ) then
		SW._units[playerName] = 0;
	end;
	
	SW._units[playerName] = SW._units[playerName] + count;
end;


function SW:ClearAuraCount( playerName )
	SW._units[playerName] = 0;
end;

function SW:GetAuraCount( playerName )
	return SW._units[playerName] or 0;
end;

function SW:Reset()
	--self:Print( "Reset" );
	SW._units = {};
	SW._playerIcons = {};
	SW._usedIcons = {};
end;

function SW:EnableAllIcons( isSlashCommand )
	local icons = self:GetEnabledIcons();
	for i,isIconUsable in ipairs(icons) do
		self:SetIsIconEnabled( i, true, isSlashCommand );
	end;
end;

function SW:DisableAllIcons( isSlashCommand )
	local icons = self:GetEnabledIcons();
	
	for i,isIconUsable in ipairs(icons) do
		self:SetIsIconEnabled( i, false, isSlashCommand );
	end;
end;


function SW:PrintStatus()
	local icons = self:GetEnabledIcons();
	
	for i,isIconUsable in ipairs(icons) do
		if( isIconUsable ) then
			self:Print( string.format("%s = ON", SW_RT2STRING[i] ) );
		else
			self:Print( string.format("%s = OFF", SW_RT2STRING[i] ) );
		end;
	end;
	
	self:Print( string.format( "Watching aura: '%s' for stack of %d", (SW_SAVED_VARIABLES.profile.watchedBuff or "-none-"), (SW_SAVED_VARIABLES.profile.buffThreshold or 1 ) ) )
end;


function SW:GetDebuff()
	return SW_SAVED_VARIABLES.profile.watchedBuff;
end;

function SW:SetDebuff( debuff, isSlashCommand )
	SW_SAVED_VARIABLES.profile.watchedBuff = debuff or "-none-";
	
	if( isSlashCommand ) then
		self:Print( string.format( "Now watching aura: '%s'", SW_SAVED_VARIABLES.profile.watchedBuff ) );
	end;
end;


function SW:GetThreshold()
	return SW_SAVED_VARIABLES.profile.buffThreshold;
end;

function SW:SetThreshold( value, isSlashCommand )
	SW_SAVED_VARIABLES.profile.buffThreshold = tonumber(value) or 0;
	
	if( isSlashCommand ) then
		self:Print( string.format( "Aura threshold set to %d", SW_SAVED_VARIABLES.profile.buffThreshold ) );
	end;
end;

function SW:IsSlashCommand( info )
	if( info[0] and strtrim( info[0] ) ~= "" ) then
		return true;
	end;
	
	return false;

end;


function SW:SetEnableStatus( isEnabled, isSlashCommand )
	SW:SetEnabledState( isEnabled );
	
	if( isEnabled ) then
		SW_SAVED_VARIABLES.profile.enabled = true;
		SW:OnEnable();
	else
		SW_SAVED_VARIABLES.profile.enabled = false;
		SW:OnDisable();
	end;
	
	if( isSlashCommand ) then
		if( isEnabled ) then
			self:Print( "Enabled" );
		else
			self:Print( "Disabled" );
		end;
	end;
end;

SW_SAVED_VARIABLES = 
{ 
	profile = 
	{
		iconsUsed = {true, true, true, true, true, true, true, true};
		watchedBuff = "",
		buffThreshold = 1,
		enabled = false,
		messagePrefix = "You have:"
	}
};


SW_SLASH_COMMANDS = 
{
    name = "StackWatch",
    handler = SW,
    type = 'group',
    args = 
    {
    	config = 
    	{
    		type = "execute",
    		name = "Config",
    		desc = "Opens the GUI configuration for StackWatch",
    		func = function()  InterfaceOptionsFrame_OpenToFrame(SW.optionsFrame); end,
    		guiHidden = true
    	},
    	status = 
    	{
    		type = "execute",
    		name = "Status",
    		desc = "List icon usage",
    		func = function() SW:PrintStatus() end,
    		guiHidden = true
    	},
    	icons = 
    	{
		    name = "Icons",
		    handler = SW,
		    type = 'group',
		    inline = true,
		    order = -1,
		    args = 
		    {
		    	star = 
		    	{
					type = "toggle",
					name = "Star",
					desc = "Toggles the use of the start raid icon",
					get = function() return SW:IsIconEnabled( 1 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 1, value, SW:IsSlashCommand(info) ) end
		    	},
		    	circle = 
		    	{
					type = "toggle",
					name = "Circle",
					desc = "Toggles the use of the circle raid icon",
					get = function() return SW:IsIconEnabled( 2 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 2, value, SW:IsSlashCommand(info) ) end
		    	},
		    	diamond = 
		    	{
					type = "toggle",
					name = "Diamond",
					desc = "Toggles the use of the diamond raid icon",
					get = function() return SW:IsIconEnabled( 3 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 3, value, SW:IsSlashCommand(info) ) end
		    	},
		    	triangle = 
		    	{
					type = "toggle",
					name = "Triangle",
					desc = "Toggles the use of the triangle raid icon",
					get = function() return SW:IsIconEnabled( 4 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 4, value, SW:IsSlashCommand(info) ) end
		    	},
		    	moon = 
		    	{
					type = "toggle",
					name = "Moon",
					desc = "Toggles the use of the moon raid icon",
					get = function() return SW:IsIconEnabled( 5 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 5, value, SW:IsSlashCommand(info) ) end
		    	},
		    	square = 
		    	{
					type = "toggle",
					name = "Square",
					desc = "Toggles the use of the square raid icon",
					get = function() return SW:IsIconEnabled( 6 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 6, value, SW:IsSlashCommand(info) ) end
		    	},
		    	cross = 
		    	{
					type = "toggle",
					name = "Cross",
					desc = "Toggles the use of the cross raid icon",
					get = function() return SW:IsIconEnabled( 7 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 7, value, SW:IsSlashCommand(info) ) end
		    	},
		    	skull = 
		    	{
					type = "toggle",
					name = "Skull",
					desc = "Toggles the use of the skull raid icon",
					get = function() return SW:IsIconEnabled( 8 ) end,
					set = function(info, value) SW:SetIsIconEnabled( 8, value, SW:IsSlashCommand(info) ) end
		    	},
		    	extended = 
		    	{
		    		name = "",
				    cmdHidden = true,
				    handler = SW,
				    type = 'group',
				    inline = true,
				    order = -1,
				    args = 
				    {
				    	enableAll = 
				    	{
				    		type = "execute",
				    		name = "Enable All",
				    		desc = "Enable all raid icons",
				    		func = function(info) SW:EnableAllIcons( SW:IsSlashCommand(info)) end,
				    		order = -2
				    	},
				    	enableNone = 
				    	{
				    		type = "execute",
				    		name = "Enable None",
				    		desc = "Disable all raid icons",
				    		func = function(info) SW:DisableAllIcons( SW:IsSlashCommand(info) ) end,
				    		order = -1
				    	}
				    }
			    }
		    }
    	},
    	
    	aura = 
    	{
    		type = "input",
    		name = "Aura",
    		desc =  "Set the aura (buff/debuff) to watch for",
    		usage = "<aura name>",
    		get = function() return SW:GetDebuff() end,
    		set = function(info, value ) SW:SetDebuff( value, SW:IsSlashCommand(info) ) end
    	},
    	threshold = 
    	{
    		type = "input",
    		name = "Threshold",
    		desc = "Set the number of stacks to warn on",
    		pattern = "[1-9]%d*",
    		usage = "<threshold>",
    		get = function() return tostring( SW:GetThreshold() ) end,
    		set = function(info, value ) SW:SetThreshold( value, SW:IsSlashCommand(info) ) end
    	},
    	message = 
    	{
    		type = "input",
    		name = "Notify Message",
    		desc = "Set the prefix message used to notify players",
    		usage = "<message>",
    		get = function() return SW:GetMessagePrefix() end,
    		set = function(info, value ) SW:SetMessagePrefix( value ) end
    	},
	    standby =
	    {
	    	name = "Enable",
	    	type = "toggle",
	    	desc = "Enable/disabled StackWatch",
	    	get = function() return SW:IsEnabled() end,
	    	set = function( info, value ) SW:SetEnableStatus( value, SW:IsSlashCommand(info) ) end,
	    	order = 1,
	    	width = "full"
	    }
    }
}
